# -*- coding: utf-8 -*-
"""
    dicts.regexpdict
    ~~~~~~~~~~~~~~~~

    Dictionary whose keys are regular expressions

    :copyright: (c) 2011 by  Miguel Olivares.
    :license: LGPL, see LICENSE for more details.
"""
from dict import Dict
import re


class RegexpDict(Dict):
    """ Keys are regular expressions. """

    def __init__(self, data={}):
        super(RegexpDict, self).__init__(data)

    def __getitem__(self, key):
        """ Returned all values that match key """
        assert(isinstance(key, basestring))
        return [v for k,v in self.iteritems() if re.match(k, key)]

    def __setitem__(self, key, value):
        assert(isinstance(key, basestring))
        return super(RegexpDict, self).__setitem__(key, value)


# -*- coding: utf-8 -*-
"""
    dicts.nocasedict
    ~~~~~~~~~~~~~~~~

    Keys of the dictionary are case insensitive.

    :copyright: (c) YEAR by  Miguel Olivares.
    :license: LICENSE_NAME, see LICENSE_FILE for more details.
"""
from dict import Dict


class NoCaseDict(Dict):
    """ Keys of the dictionary are case insensitive. """

    def __init__(self, data):
        """
        :param data: dictionary
        """
        correct_keys = reduce(lambda x, y: x and isinstance(y, basestring),
                              data.iterkeys(), True)
        if not correct_keys:
            raise KeyError
        lower_dict = dict(map(lambda x : (x[0].lower(), x[1]), data.items()))
        super(NoCaseDict, self).__init__(lower_dict)

    def __setitem__(self, key, value):
        if not isinstance(key, basestring):
            raise KeyError
        super(NoCaseDict, self).__setitem__(key.lower(), value)

    def __contains__ (self, key):
        if not isinstance(key, basestring):
            raise KeyError
        return super(NoCaseDict, self).__contains__(key.lower())


# -*- coding: utf-8 -*-


import unittest
from dicts import NoCaseDict



class DictTestCase(unittest.TestCase):
    """ Dict features test cases."""

    def setUp(self):
        self.d = NoCaseDict({'KeY1' : 1})

    def tearDown(self):
        pass

    def test_contains(self):
        self.assertIn('key1', self.d)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-


import unittest
from dicts import Dict



class DictTestCase(unittest.TestCase):
    """ Dict features test cases."""

    def setUp(self):
        self.d = Dict({1 : 7, 2 : [1,2], 3 : 'a'})

    def tearDown(self):
        pass

    def test_join(self):
        self.d.join(Dict({1 : 2, 2 : [3], 3 : 'b'}))
        self.assertEqual(self.d.items(), [(1, 9), (2, [1, 2, 3]), (3, 'ab')])

    def test_lookup(self):
        d = Dict()
        d['1'] = 10
        d['2'] = 20
        d['a'] = 30
        self.assertEqual(d.relookup('\d+'), [('1', 10), ('2', 20)])

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
"""
    dicts.dict
    ~~~~~~~~~~

    Extends UserDict to implement some awesome features

    :copyright: (c) 2011 by  Miguel Olivares <miguel@moliware.com>.
    :license: LGPL, see LICENSE for more details.
"""
import re
from copy import deepcopy
from UserDict import UserDict


class Dict(UserDict, object):
    """ Iterable UserDict. It is thought for being
    the base class of all specific dictionaries.
    """

    def __iter__(self):
        """ Iterable dictionary! """
        return self.data.iteritems()

    def join(self, dic):
        """ Add dic pairs to self.data """
        for k,v in dic.iteritems():
            if k in self.data:
                self[k] += dic[k]
            else:
                self[k] = deepcopy(v)
        return self

    def map(self, callable):
        """ Apply 'callable' function over all values. """
        for k,v in self.iteritems():
            self[k] = callable(v)

    @classmethod
    def fromrepetitions(cls, iterable):
        """ Create a dict whose keys are the members of the iterable and values
        are the number of times the key appears in the iterable.
        """
        d = cls()
        for key in iterable:
            d[key] = d[key] + 1 if key in d else 1
        return d

    def relookup(self, pattern):
        """ Dictionary lookup with a regular expression. Return pairs whose key 
        matches pattern.
        """
        key = re.compile(pattern)
        return filter(lambda x : key.match(x[0]), self.data.items())